#
# tilingtest.py: Test tiling.C.
# 
# Copyright (C) 2016-2018, David Moews.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import subprocess
import re
import sys

class TestFileError(Exception):
	pass

def is_ok(v):
	return 'OK' if v else 'not OK'

def split_space(x):
	return [] if (x == '') else x.split(' ')

if (len(sys.argv) >= 2):
	command_name = sys.argv[1].split(' ')
else:
	command_name = ['./tiling4']

if (len(sys.argv) >= 3):
	test_name = sys.argv[2]
else:
	test_name = 'tilingtest.test'


if (len(sys.argv) >= 4):
	usage_name = sys.argv[3]
else:
	usage_name = 'tilingtest.usage'

test_failed = False

# Trim off final newline.
usage = open(usage_name).read()[:-1]
usage = command_name[0].join(usage.split('@'))

flag_choices = ['']
flag_flags = ['1']

testfile = open(test_name).read()

while (testfile != ''):
	i = testfile.index('\n')
	if (testfile[:1] == '#'):
		testfile = testfile[i+1:]
	elif (testfile[:5] == 'FLAGS'):
		flag_choices = testfile[6:i].split(';')
		testfile = testfile[i+1:]
	elif (testfile[:5] == 'FLAGC'):
		flag_flags = testfile[6:i].split(';')
		testfile = testfile[i+1:]
	elif (testfile[:2] == 'IN'):
		arglist = split_space(testfile[3:i])
		testfile = testfile[i+1:]
		desired_output_lists = [[] for x in flag_choices]
		desired_errout_lists = [[] for x in flag_choices]
		while (testfile[:2] == 'OU') or (testfile[:2] == 'ER') or (testfile[:1] == '#'):
			i = testfile.index('\n')
			if (testfile[:1] == '#'):
				testfile = testfile[i+1:]
				continue
			piece = testfile[4:i]
			if (piece == '$USAGE'):
				piece = usage
			if (testfile[:3] == 'OUT'):
				for x in desired_output_lists:
					x.extend([piece, '\n'])
			elif (testfile[:2] == 'OU'):
				for desired_output_list, flag in zip(desired_output_lists, flag_flags):
					if (flag == testfile[2]):
						desired_output_list.extend([piece, '\n'])
			elif (testfile[:3] == 'ERR'):
				for x in desired_errout_lists:
					x.extend([piece, '\n'])
			elif (testfile[:2] == 'ER'):
				for desired_errout_list, flag in zip(desired_errout_lists, flag_flags):
					if (flag == testfile[2]):
						desired_errout_list.extend([piece, '\n'])
			testfile = testfile[i+1:]
		for flag_choice, desired_output_list, desired_errout_list in zip(flag_choices, desired_output_lists, desired_errout_lists):
			cmdline = command_name + split_space(flag_choice) + arglist
			p = subprocess.Popen(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			(raw_output, errout) = p.communicate()
			split_output = raw_output.split('\n')
			output_list = []
			for l in split_output:
				if not re.match('^(Setup|Search) time: [0-9]* ticks rdtsc, [0-9]* ns real, [0-9]* ns cpu this process\.$', l):
					output_list.append(l)
			output = '\n'.join(output_list)
			desired_output = ''.join(desired_output_list)
			desired_errout = ''.join(desired_errout_list)
			if (output != desired_output) or (errout != desired_errout):
				print 'Command', ' '.join(cmdline), 'output', is_ok(output == desired_output), 'error', is_ok(errout == desired_errout)
				if (output != desired_output):
					print 'Output:'
					print output
					print 'Desired output:'
					print desired_output
				if (errout != desired_errout):
					print 'Error output:'
					print errout
					print 'Desired error output:'
					print desired_errout
				test_failed = True
	else:
		print 'Error in test file'
		raise TestFileError()

if (not test_failed):
	print 'All tests OK'
